Package: surveillance
Title: Temporal and Spatio-Temporal Modeling and Monitoring of Epidemic
        Phenomena
Version: 1.26.1
Date: 2026-07-30
Authors@R: c(
    MH = person("Michael", "Hoehle",
                role = c("aut", "ths"),
                comment = c(ORCID = "0000-0002-0423-6702")),
    SM = person("Sebastian", "Meyer",
                email = "seb.meyer@fau.de",
                role = c("aut", "cre"),
                comment = c(ORCID = "0000-0002-1791-9449")),
    MP = person("Michaela", "Paul",
                role = "aut"),
    LH = person("Leonhard", "Held",
                role = c("ctb", "ths"),
                comment = c(ORCID = "0000-0002-8686-5325")),
    person("Howard", "Burkom", role = "ctb"),
    person("Thais", "Correa", role = "ctb"),
    person("Mathias", "Hofmann", role = "ctb"),
    person("Christian", "Lang", role = "ctb"),
    person("Juliane", "Manitz", role = "ctb"),
    person("Sophie", "Reichert", role = "ctb"),
    person("Andrea", "Riebler", role = "ctb"),
    person("Daniel", "Sabanes Bove", role = "ctb"),
    MS = person("Maelle", "Salmon", role = "ctb"),
    DS = person("Dirk", "Schumacher", role = "ctb"),
    person("Stefan", "Steiner", role = "ctb"),
    person("Mikko", "Virtanen", role = "ctb"),
    person("Wei", "Wei", role = "ctb"),
    person("Valentin", "Wimmer", role = "ctb"),
    person("R Core Team", role = "ctb",
           comment = c(ROR = "02zz1nj61",
                       "src/ks.c and a few code fragments of standard S3 methods"))
    )
Depends: R (>= 3.6.0), methods, grDevices, graphics, stats, utils, sp
        (>= 2.1-4)
Imports: polyCub (>= 0.8.0), MASS, Matrix, nlme, spatstat.geom
LinkingTo: polyCub
Suggests: parallel, grid, xtable (>= 1.7-0), gridExtra (>= 2.0.0),
        lattice (>= 0.20-44), colorspace, scales, animation, msm, spc,
        coda, runjags, INLA, spdep, numDeriv, maxLik, gsl, fanplot,
        hhh4contacts, quadprog, memoise, polyclip, intervals, splancs,
        gamlss, MGLM (>= 0.1.0), sf, tinytest (>= 1.4.1), knitr, bibtex
Enhances: xts, ggplot2
Description: Statistical methods for the modeling and monitoring of time series
    of counts, proportions and categorical data, as well as for the modeling
    of continuous-time point processes of epidemic phenomena.
    The monitoring methods focus on aberration detection in count data time
    series from public health surveillance of communicable diseases, but
    applications could just as well originate from environmetrics,
    reliability engineering, econometrics, or social sciences. The package
    implements many typical outbreak detection procedures such as the
    (improved) Farrington algorithm, or the negative binomial GLR-CUSUM
    method of Hoehle and Paul (2008) <doi:10.1016/j.csda.2008.02.015>.
    A novel CUSUM approach combining logistic and multinomial logistic
    modeling is also included. The package contains several real-world data
    sets, the ability to simulate outbreak data, and to visualize the
    results of the monitoring in a temporal, spatial or spatio-temporal
    fashion. A recent overview of the available monitoring procedures is
    given by Salmon et al. (2016) <doi:10.18637/jss.v070.i10>.
    For the retrospective analysis of epidemic spread, the package provides
    three endemic-epidemic modeling frameworks with tools for visualization,
    likelihood inference, and simulation. hhh4() estimates models for
    (multivariate) count time series following Paul and Held (2011)
    <doi:10.1002/sim.4177> and Meyer and Held (2014) <doi:10.1214/14-AOAS743>.
    twinSIR() models the susceptible-infectious-recovered (SIR) event
    history of a fixed population, e.g, epidemics across farms or networks,
    as a multivariate point process as proposed by Hoehle (2009)
    <doi:10.1002/bimj.200900050>. twinstim() estimates self-exciting point
    process models for a spatio-temporal point pattern of infective events,
    e.g., time-stamped geo-referenced surveillance data, as proposed by
    Meyer et al. (2012) <doi:10.1111/j.1541-0420.2011.01684.x>.
    A recent overview of the implemented space-time modeling frameworks
    for epidemic phenomena is given by Meyer et al. (2017)
    <doi:10.18637/jss.v077.i11>.
License: GPL-2
URL: https://surveillance.R-Forge.R-project.org/
Additional_repositories: https://inla.r-inla-download.org/R/stable/
VignetteBuilder: utils, knitr
Note: Building the package requires R >= 4.6.0 for Rd \bibshow{} et al.
NeedsCompilation: yes
Packaged: 2026-07-30 14:16:39 UTC; smeyer
Author: Michael Hoehle [aut, ths] (ORCID:
    <https://orcid.org/0000-0002-0423-6702>),
  Sebastian Meyer [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1791-9449>),
  Michaela Paul [aut],
  Leonhard Held [ctb, ths] (ORCID:
    <https://orcid.org/0000-0002-8686-5325>),
  Howard Burkom [ctb],
  Thais Correa [ctb],
  Mathias Hofmann [ctb],
  Christian Lang [ctb],
  Juliane Manitz [ctb],
  Sophie Reichert [ctb],
  Andrea Riebler [ctb],
  Daniel Sabanes Bove [ctb],
  Maelle Salmon [ctb],
  Dirk Schumacher [ctb],
  Stefan Steiner [ctb],
  Mikko Virtanen [ctb],
  Wei Wei [ctb],
  Valentin Wimmer [ctb],
  R Core Team [ctb] (ROR: <https://ror.org/02zz1nj61>, src/ks.c and a few
    code fragments of standard S3 methods)
Maintainer: Sebastian Meyer <seb.meyer@fau.de>
Repository: CRAN
Date/Publication: 2026-07-30 17:10:02 UTC
