\name{primeFactors}
\alias{primeFactors}
\title{Prime Number Factorization}
\description{
  Computes the prime number factorization of an integer.
}
\usage{
primeFactors(x)
}
\arguments{
  \item{x}{an integer}
}
\value{vector with prime number factorization of \code{x}}
\keyword{math}
