\name{twinstim_plot}
\alias{plot.twinstim}

\title{
Plots for a Fitted \code{twinstim}
}

\description{
The fitted conditional intensity function from \code{\link{twinstim}}
may be visualized in at least two ways: \code{\link{iafplot}} plots the fitted
interaction functions (as a function of the distance from the host), and
\code{\link{intensityplot.twinstim}} plots the fitted intensity either
aggregated over space (evolution over time) or aggregated over time
(spatial surface of the cumulated intensity). The \code{plot} method for
class \code{"twinstim"} is just a wrapper for these two functions.
}

\usage{
\method{plot}{twinstim}(x, which, ...)
}

\arguments{
  \item{x}{
    an object of class \code{"twinstim"}.
  }
  \item{which}{
    character. Which characteristic of the conditional intensity should
    be plotted? Possible values are the ones allowed in
    the functions \code{\link{iafplot}} (e.g., \code{"siaf"}) and
    \code{\link{intensityplot.twinstim}} (e.g.,
    \code{"epidemic proportion"}, partially matched).
  }
  \item{\dots}{
    further arguments passed to \code{iafplot} or
    \code{intensityplot.twinstim}.
  }
}

\value{
See the documentation of the respective plot functions,
\code{\link{iafplot}} or \code{\link{intensityplot.twinstim}}.
}

\author{
Sebastian Meyer
}

\keyword{hplot}
